@@ -1,7 +1,7 @@
 /*
 
 Copyright (C) 2002 Hayato Fujiwara <h_fujiwara@users.sourceforge.net>
-Copyright (C) 2010-2015 Olaf Till <i7tiol@t-online.de>
+Copyright (C) 2010-2016 Olaf Till <i7tiol@t-online.de>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -23,14 +23,12 @@ along with this program; If not, see <http://www.gnu.org/licenses/>.
 
 #include <octave/oct.h>
 #include <octave/symtab.h>
-
-#include <oct-env.h>
-
-#include <file-io.h>
-#include <sighandlers.h>
-#include <parse.h>
-#include <cmd-edit.h>
-#include <toplev.h>
+#include <octave/oct-env.h>
+#include <octave/file-io.h>
+#include <octave/sighandlers.h>
+#include <octave/parse.h>
+#include <octave/cmd-edit.h>
+#include <octave/toplev.h>
 
 #include <sys/socket.h>
 #include <sys/types.h>
@@ -140,16 +138,16 @@ reval_loop (octave_parallel_stream &cstr)
           dsprintf ("reval loop, after successful network_recv_string\n");
         }
 
+      bool err;
       int p_err;
-      eval_string (s, false, p_err, 0);
+      SET_ERR (eval_string (s, false, p_err, 0), err);
       dsprintf ("reval loop, after evaluating string\n");
 
       uint32_t nl = 0;
-      if (error_state)
+      if (err)
         {
-          dsprintf ("reval loop, error state was set\n");
+          dsprintf ("reval loop, evaluation caused error\n");
           nl = 1;
-          error_state = 0;
         }
       else if (p_err)
         {
@@ -352,15 +350,9 @@ the swap device even after shutdown of the application.\n\
     {
       octave_scalar_map options;
 
-      options = args(0).scalar_map_value ();
-
-      if (error_state)
-        {
-          error ("%s: could not convert second argument to scalar structure",
-                 fname.c_str ());
-
-          return retval;
-        }
+      CHECK_ERROR (options = args(0).scalar_map_value (), retval,
+                   "%s: could not convert second argument to scalar structure",
+                   fname.c_str ());
 
       octave_value tmp;
 
@@ -369,15 +361,9 @@ the swap device even after shutdown of the application.\n\
 
       if (tmp.is_defined ())
         {
-          use_gnutls = tmp.bool_value ();
-
-          if (error_state)
-            {
-              error ("%s: could not convert option 'use_tls' to bool",
-                     fname.c_str ());
-
-              return retval;
-            }
+          CHECK_ERROR (use_gnutls = tmp.bool_value (), retval,
+                       "%s: could not convert option 'use_tls' to bool",
+                       fname.c_str ());
         }
 
       // custom authentication file
@@ -385,15 +371,9 @@ the swap device even after shutdown of the application.\n\
 
       if (tmp.is_defined ())
         {
-          cafile = tmp.string_value ();
-
-          if (error_state)
-            {
-              error ("%s: could not convert option 'auth_file' to string",
-                     fname.c_str ());
-
-              return retval;
-            }
+          CHECK_ERROR (cafile = tmp.string_value (), retval,
+                       "%s: could not convert option 'auth_file' to string",
+                       fname.c_str ());
         }
     } // args.length () == 1
 
@@ -634,20 +614,13 @@ the swap device even after shutdown of the application.\n\
                     {
                       octave_value_list f_args (1);
                       f_args(0) = octave_value ("prefix");
-                      octave_value_list f_ret = feval ("pkg", f_args, 1);
-                      if (error_state)
-                        {
-                          error ("%s: could not get prefix from pkg",
-                                 fname.c_str ());
-                          _exit (1);
-                        }
-                      cafile = f_ret(0).string_value ();
-                      if (error_state)
-                        {
-                          error ("%s: could not convert output of pkg ('prefix') to string)",
-                                 fname.c_str ());
-                          _exit (1);
-                        }
+                      octave_value_list f_ret;
+                      CHECK_ERROR_EXIT1 (f_ret = feval ("pkg", f_args, 1),
+                                         "%s: could not get prefix from pkg",
+                                         fname.c_str ());
+                      CHECK_ERROR_EXIT1 (cafile = f_ret(0).string_value (),
+                                         "%s: could not convert output of pkg ('prefix') to string)",
+                                         fname.c_str ());
                       cafile = cafile + file_ops::dir_sep_str () +
                         "parallel-srp-data" + file_ops::dir_sep_str () +
                         file_ops::dir_sep_str () + "server" +
@@ -696,7 +669,7 @@ the swap device even after shutdown of the application.\n\
             }
           else
 #endif
-            cmd_strb = new octave_parallel_socket_streambuf (asock);
+            cmd_strb = new octave_parallel_socket_streambuf (asock, true);
           octave_parallel_stream cmd_str (cmd_strb);
           if (! cmd_str.good ())
             {
@@ -887,7 +860,7 @@ the swap device even after shutdown of the application.\n\
 #endif
                     conn->insert_data_stream
                       (new octave_parallel_stream
-                       (new octave_parallel_socket_streambuf (adsock)));
+                       (new octave_parallel_socket_streambuf (adsock, true)));
 
                   __sockg.__release ();
 
@@ -934,10 +907,10 @@ the swap device even after shutdown of the application.\n\
                   else
                     {
                       conn->get_data_stream ()->network_send_4byteint (0);
-                      conn->connection_read_header ();
-                      write_header (conn->get_data_stream ()->get_ostream (),
-                                    LS_BINARY);
-                      if (error_state || ! conn->get_data_stream ()->good ())
+                      int err = conn->connection_read_header ();
+                      minimal_write_header
+                        (conn->get_data_stream ()->get_ostream ());
+                      if (err || ! conn->get_data_stream ()->good ())
                         {
                           error ("communication error in initialization");
                           _exit (1);
@@ -1000,7 +973,7 @@ the swap device even after shutdown of the application.\n\
               if (getaddrinfo (hosts(i - 1).c_str (), "12501", &hints, &ai))
                 {
                   error ("getaddrinfo returned an error");
-                  return retval;
+                  _exit (1);
                 }
               struct __aiguard
               {
@@ -1048,7 +1021,7 @@ the swap device even after shutdown of the application.\n\
 #endif
                         conn->insert_data_stream
                           (new octave_parallel_stream
-                           (new octave_parallel_socket_streambuf (dsock)));
+                           (new octave_parallel_socket_streambuf (dsock, false)));
                       __sockg.__release ();
                       if (! conn->get_data_stream ()->good ())
                         {
@@ -1092,11 +1065,9 @@ the swap device even after shutdown of the application.\n\
                         }
                       else
                         {
-                          write_header
-                            (conn->get_data_stream ()->get_ostream (),
-                             LS_BINARY);
-                          conn->connection_read_header ();
-                          if (error_state ||
+                          minimal_write_header
+                            (conn->get_data_stream ()->get_ostream ());
+                          if (conn->connection_read_header () ||
                               ! conn->get_data_stream ()->good ())
                             {
                               error ("communication error in initialization");
